using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// DustEditor v1.0
//
// Allows you to edit some parameters of dust particles in the package Dust 101 v2

namespace DustEditorPlugin
{
    public class DustEditor : MVRScript
    {
		public JSONStorableColor DustMainColor;
		public JSONStorableFloat DustMainOpacity;
		public Transform ParticleRoot;
		private ParticleSystem prtComponent;
		private string particleSuffixName = "_editable";

		private bool deInitialized = false;
		
		// Security
		private int initLoops = 0;
		private int maxInitLoops = 3600;

		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add DustEditor on an Custom Unity Asset with a compatible particle system.");
                    return;
                }
				
				HSVColor hsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				DustMainColor = new JSONStorableColor("Particles Color", hsvc, MainColorCallback);
				RegisterColor(DustMainColor);
				CreateColorPicker(DustMainColor);
				
				DustMainOpacity = new JSONStorableFloat("Particles Opacity", 1.0f, MainOpacityCallback, 0f, 1f);
				RegisterFloat(DustMainOpacity);
				CreateSlider(DustMainOpacity, false);
				
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>DustEditor</b> is a tiny plugin that allows you to edit some parameters of the dust particles inside Dust 101 v2.\n\n" +
					"Just test the parameters on the left, and the particle will update in realtime.\n\n" +
					"Don't be eager to see the changes! Particles have a lifetime of around 20 seconds each, the change will progressively appear after their lifecycle ends.\n\n" +
					"Same when you load a scene, the settings stored in your scene will take a few moments to apply as it is also tied to particles lifecycle.\n\n" +
					"<b>For creators and screenshot/still artists :</b> Be careful. I cannot explain why, but the vanilla screenshot feature of VAM does not render exactly the same as what you see on the realtime viewport. " +
					"In my case, I used virtual resolution to upscale the game and make higher res screenshots.\n\n" +
					"<b>IMPORTANT :</b> If you switch particles in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 1100.0f;
				

			}
            catch(Exception e)
            {
                SuperController.LogError("VAMLock - Exception caught: " + e);
            }
		}
		
		void Start()
        {

        }
		
		void Update()
        {
			// If we haven't made over 1min of init tests, we aren't initialized and the particle system (CUA) is still compatible, try to initialize
			if( initLoops < maxInitLoops && deInitialized == false ) {
				initLoops++;
				InitDE();
			}
			
			if( initLoops == maxInitLoops ) {
				logError("DustEditor : This CUA particle could not be initialized properly after 3600 tries. Please select a compatible particle and reload the script.");
			}
		}
		
		private bool InitDE() {
			if( deInitialized == true ) return true;
			
			ParticleRoot = getParticleRoot( containingAtom.gameObject.transform );
			if( ParticleRoot == null ) {
				return false;
			}
			
			prtComponent = ParticleRoot.GetComponent<ParticleSystem>();
			if( prtComponent ) {		
				deInitialized = true;
				
				MainColorCallback( DustMainColor );
				MainOpacityCallback( DustMainOpacity.val );
				
				logDebug("DustEditor : Particle editor initialized properly");
							
				return true;
			} else {
				logError("DustEditor : your CUA has a transform with _editable but does not have a ParticleSystem component attached.");
				initLoops = maxInitLoops; // We don't have a particle system, we consider the initialization failed
				return false;
			}
		}
		
		private Transform getParticleRoot( Transform parent ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(particleSuffixName) ) {
					return child;
				} else {
					return getParticleRoot( child );
				}
			}
			
			return null;
		}
		
		protected void MainColorCallback(JSONStorableColor color) {
			if( prtComponent ) {
				Color newColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, DustMainOpacity.val);
				var prtMain = prtComponent.main;
				prtMain.startColor = newColor;
			}
		}
		
		protected void MainOpacityCallback(float opacity) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startColor = new Color(DustMainColor.colorPicker.currentColor.r, DustMainColor.colorPicker.currentColor.g, DustMainColor.colorPicker.currentColor.b, opacity);
			}
		}
		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
	}
}